% Ellipses
%
% The program describes the motion of precession of the ellipse when the
% phase in linearly increasing at every rotation by a given amount. If the
% phase increment is the submultiple N of pi, after N rotations the ellipse
% returns to the initial condition, describing a periodic motion.
%
clear all;
global NTS
NTS=512;   % Number of points in a period T = Number of points in a rotation of 2*pi
NSYM=1024;   % 2*NSYM= number of rotations - NSYM rotations on every time semiaxis
NTL=min(NSYM,NSYM); % Number of rotations shown on the negative time axis (left)
NTR=min(NSYM,NSYM); % Number of rotations shown on the positive time axis (right)
NB=7;
%
freq=(-NTS/2:1/(2*NSYM):(NTS-1/NSYM)/2); % Normalized frequency axis. Normalization B=1/T
findex=(NTS*NSYM+1:NTS*NSYM+1+NB*2*NSYM);
fplot=freq(findex); % Unilateral normalized frequency axis for plotting functions
bindex=(NTS*NSYM+1-NB*2*NSYM:NTS*NSYM+1+NB*2*NSYM);
bplot=freq(bindex); % Bilateral normalized frequency axis for plotting functions
time=(-NSYM:1/NTS:NSYM-1/NTS); % Normalized time axis. Normalization T
tindex=(NTS*NSYM+1-NTL*NTS:NTS*NSYM+NTR*NTS); % Index vector of the temporal axis
tplot=time(tindex); % Normalized temporal axis used for plotting the output pulse
%
%--------------------------------------------------------------------------
%
% The time step T coincides with the period of rotation
%
a=1;                            % Major axis
b=0.2;                          % Minor axis
AR0=max(a,b)/min(a,b);          % Axial ratio
ecc=sqrt(1-1/AR0^2);            % Eccentricity
rot=+1;                         % Direction of rotation: +1=counterclockwise, -1=clockwise
psi=0;                          % Phase offset
tilt=pi/3;                      % Tilt angle
alpha=2*pi*time;                % Angular variable -2*pi*NSZM:+2*pi*NSZM
%
% Perturbations
%
Delta_tilt_deg=0;                % Tilt angle increment in degree after a single rotation.
Delta_tilt=Delta_tilt_deg*pi/180; % Tilt angle increment in radiant after a single rotation.
Delta_AR_rel=9e-4;               % Relative axial ratio increment after a single rotation
Delta_AR=Delta_AR_rel*AR0;        % Axial ratio increment after a single rotation.
%
% Inizialization
%
theta=zeros(1,length(alpha));
rho=zeros(1,length(alpha));
AR=zeros(1,length(alpha));
rho_2=zeros(1,length(alpha));
x=zeros(1,length(alpha));
y=zeros(1,length(alpha));
phi=zeros(1,length(alpha));
%
% Calculation
%
for k=1:length(alpha),
    AR(k)=AR0+(k-(NTS*NSYM+1))*Delta_AR/NTS;
    theta(k)=tilt+(k-(NTS*NSYM+1))*Delta_tilt/NTS;
    x(k)=cos(alpha(k)+psi);
    y(k)=rot/AR(k)*sin(alpha(k)+psi);
    phi(k)=theta(k)+atan2(y(k),x(k));
    rho(k)=1./sqrt((cos(phi(k))^2*(cos(theta(k))^2+AR(k)^2*sin(theta(k))^2)+...
        sin(phi(k))^2*(sin(theta(k))^2+AR(k)^2*cos(theta(k))^2)-...
        (AR(k)^2-1)/2*sin(2*phi(k))*sin(2*theta(k))));
    rho_2(k)=sqrt((1+1/AR(k)^2)/2+(1-1/AR(k)^2)/2*cos(2*(alpha(k)+psi)));
end;
%
% Angular velocity
%
deriv_phi_anal=Delta_tilt+((AR-Delta_AR/(4*pi)*sin(2*(alpha+psi)))./(AR.^2.*cos(alpha+psi).^2+sin(alpha+psi).^2));
deriv_phi=diff(unwrap(phi))*NTS/(2*pi);
deriv_phi=[deriv_phi, deriv_phi(length(deriv_phi))];
%
% Spectrum
%
spectrum_field_x=SPECTRUM(x)/NSYM;
spectrum_field_y=SPECTRUM(y)/NSYM;
spectrum_modulus=SPECTRUM(rho_2);
spectrum_modulus=spectrum_modulus/max(spectrum_modulus)+1e-300;
spectrum_intensity=SPECTRUM(rho_2.^2);
spectrum_intensity=spectrum_intensity/max(spectrum_intensity)+1e-300;
%
% Graphics
%
figure(1);
clf;
subplot(2,2,1);
polar(phi,rho);
hold on;
polar(phi(NTS*NSYM+1),rho(NTS*NSYM+1),'ro');
polar(phi(NTS*NSYM+1:NTS*NSYM+1+NTS),rho(NTS*NSYM+1:NTS*NSYM+1+NTS),'r');
polar(phi(NTS*NSYM+1+NTS),rho(NTS*NSYM+1+NTS),'rs');
hold off;
title(['\theta=',num2str(tilt*180/pi),', \Delta\theta=',num2str(Delta_tilt*180/pi,'%3.2f'),...
    ', \rho=' num2str(AR0),', \Delta\rho=',num2str(Delta_AR),', \psi=',num2str(psi*180/pi),'']);
subplot(2,2,2);
plot(alpha(tindex)/(2*pi),phi(tindex)/pi,'b','Linewidth',2);
xlabel('Normalized time t/T');
ylabel('Angular position \phi/\pi');
title(['Angular position vs. time: \theta=',num2str(tilt*180/pi),...
    ', \Delta\theta=',num2str(Delta_tilt*180/pi,'%3.2f'),', \rho=' num2str(AR0),...
    ', \Delta\rho=',num2str(Delta_AR),', \psi=',num2str(psi*180/pi),'']);
grid on;
subplot(2,2,3);
plot(alpha(tindex)/(2*pi),rho(tindex),alpha(tindex)/(2*pi),rho_2(tindex),'Linewidth',2);
xlabel('Normalized time t/T');
ylabel('Modulus E(t)');
title(['Electric field modulus vs. time: \theta=',num2str(tilt*180/pi),...
    ', \Delta\theta=',num2str(Delta_tilt*180/pi,'%3.2f'),', \rho=' num2str(AR0),...
    ', \Delta\rho=',num2str(Delta_AR),', \psi=',num2str(psi*180/pi),'']);
grid on;
subplot(2,2,4);
if (Delta_tilt==0) && (Delta_AR==0),
    plot(alpha(tindex)/(2*pi),deriv_phi_anal,alpha/(2*pi),deriv_phi(tindex),'Linewidth',2);
else
    plot(alpha(tindex)/(2*pi),deriv_phi(tindex),'Linewidth',2);
end;
grid on;
xlabel('Normalized time t/T');
ylabel('Angular velocity d\phi/dt [1/\omega]');
title(['Angular velocity vs. time: \theta=',num2str(tilt*180/pi),...
    ', \Delta\theta=',num2str(Delta_tilt*180/pi,'%3.2f'),', \rho=' num2str(AR0),...
    ', \Delta\rho=',num2str(Delta_AR),', \psi=',num2str(psi*180/pi),'']);
%
figure(2);
clf;
subplot(1,2,1);
polar(phi,rho);
hold on;
polar(phi(NTS*NSYM+1),rho(NTS*NSYM+1),'o');
hold off;
title(['\theta=',num2str(tilt*180/pi),...
    ', \Delta\theta=',num2str(Delta_tilt*180/pi,'%3.2f'),', \rho=' num2str(AR0),...
    ', \Delta\rho=',num2str(Delta_AR),', \psi=',num2str(psi*180/pi),'']);
subplot(1,2,2);
plot(alpha(tindex)/(2*pi),rho(tindex),'Linewidth',2);
xlabel('Normalized time t/T');
ylabel('Intensity');
title(['Electric field modulus vs. time: \theta=',num2str(tilt*180/pi),...
    ', \Delta\theta=',num2str(Delta_tilt*180/pi,'%3.2f'),', \rho=' num2str(AR0),...
    ', \Delta\rho=',num2str(Delta_AR),', \psi=',num2str(psi*180/pi),'']);
grid on;
%
figure(3);
subplot(411);
plot(bplot,20*log10(abs(spectrum_field_x(bindex))),'Linewidth',2,'Color','b');
grid on;
xlabel('Normalized frequency [1/\omega_0]');
ylabel('Field spectrum [dB]');
title(['X-field spectrum: \theta=',num2str(tilt*180/pi),...
    ', \Delta\theta=',num2str(Delta_tilt*180/pi,'%3.2f'),', \rho=' num2str(AR0),...
    ', \Delta\rho=',num2str(Delta_AR),', \psi=',num2str(psi*180/pi),'']);
subplot(412);
plot(bplot,20*log10(abs(spectrum_field_y(bindex))),'Linewidth',2,'Color','r');
grid on;
xlabel('Normalized frequency [1/\omega_0]');
ylabel('Field spectrum [dB]');
title(['Y-field spectrum: \theta=',num2str(tilt*180/pi),...
    ', \Delta\theta=',num2str(Delta_tilt*180/pi,'%3.2f'),', \rho=' num2str(AR0),...
    ', \Delta\rho=',num2str(Delta_AR),', \psi=',num2str(psi*180/pi),'']);
subplot(413);
plot(bplot,20*log10(abs((spectrum_modulus(bindex)))),'Linewidth',2,'Color','b');
grid on;
xlabel('Normalized frequency [1/\omega_0]');
ylabel('Modulus spectrum [dB]');
title(['Modulus spectrum: \theta=',num2str(tilt*180/pi),...
    ', \Delta\theta=',num2str(Delta_tilt*180/pi,'%3.2f'),', \rho=' num2str(AR0),...
    ', \Delta\rho=',num2str(Delta_AR),', \psi=',num2str(psi*180/pi),'']);
subplot(414);
plot(bplot,10*log10(abs(spectrum_intensity(bindex))),'Linewidth',2,'Color','b');
grid on;
xlabel('Normalized frequency [1/\omega_0]');
ylabel('Intensity spectrum [dB]');
title(['Intensity spectrum: \theta=',num2str(tilt*180/pi),...
    ', \Delta\theta=',num2str(Delta_tilt*180/pi,'%3.2f'),', \rho=' num2str(AR0),...
    ', \Delta\rho=',num2str(Delta_AR),', \psi=',num2str(psi*180/pi),'']);
